import numpy as np

arcsec_in_radian = 1 / 3600 / 180 * np.pi
arcsec_in_degrees = 1 / 3600

degrees_in_arcsec = 3600
radian_in_arcsec = 3600 * 180 * np.pi

uradian_in_arcsec = (3600 * 180) / 1e6 / np.pi



from geo3d import Point
from geo3d.query import distance_between_points, minimize_points_to_points_distance

import pytest


def test_distance_between_points():
    pointA = Point([1, 2, 3])
    pointB = Point([1, 2, 6])
    assert distance_between_points(pointA, pointB) == 3


def test_minimize_points_to_points_distance():
    groupA = [[1, 0, 0], [0, 1, 0], [-1, 0, 0], [0, -1, 0], [0, 0, 1], [0, 0, -1]]
    groupB = [[0, 1, 0], [-1, 0, 0], [0, -1, 0], [1, 0, 0], [0, 0, 1], [0, 0, -1]]
    trafo = minimize_points_to_points_distance(groupA, groupB)
    assert trafo.translation.as_array() == pytest.approx([0, 0, 0], abs=1e-8)
    assert trafo.intrinsic_euler_angles() == pytest.approx([0, 0, 90], abs=1e-6)



import pytest

from geo3d import frame_wizard, Vector

@pytest.fixture
def example_frames():
    # rotation only from UnitFrame
    fa = frame_wizard(Vector([1, 1, 0]), Vector([1, -1, 0]), "x", "y", origin=[0, 0, 0])
    # translation only from UnitFrame
    fb = frame_wizard(Vector([1, 0, 0]), Vector([0, 1, 0]), "x", "y", origin=[1, 1, 4])
    # rotation and translation from UnitFrame
    fc = frame_wizard(Vector([1, 1, 0]), Vector([1, -1, 0]), "x", "y", origin=[1, 1, 4])
    return fa, fb, fc


import pytest

from geo3d import Point
import numpy as np

def test_point_from_array():
    p = Point([1,2,3])
    assert p[2]==3

def test_point_from_tuple():
    p = Point((1,2,3))
    assert p[2]==3

def test_point_from_np():
    p = Point(np.array([1,2,3]))
    assert p[2]==3

def test_point_from_np_bare_nocopy():
    a = np.array([1,2,3])
    p = Point.from_array(a, copy=False)
    a[2]=7
    assert p[2]==7

def test_point_from_np_bare_copy():
    a = np.array([1,2,3])
    p = Point.from_array(a, copy=True)
    a[2]=7
    assert p[2]==3


import pytest

from geo3d import Vector
import numpy as np


def test_vector_from_array():
    p = Vector([1, 2, 3])
    assert p[2] == 3


def test_vector_from_tuple():
    p = Vector((1, 2, 3))
    assert p[2] == 3


def test_vector_from_np():
    p = Vector(np.array([1, 2, 3]))
    assert p[2] == 3


def test_vector_from_np_bare_nocopy():
    a = np.array([1, 2, 3])
    p = Vector.from_array(a, copy=False)
    a[2] = 7
    assert p[2] == 7


def test_vector_from_np_bare_copy():
    a = np.array([1, 2, 3])
    p = Vector.from_array(a, copy=True)
    a[2] = 7
    assert p[2] == 3



import pytest
from geo3d import Frame, frame_wizard, Point, Vector, UnitFrame, RotationMatrix
import math

def test_point_express_in_frame(example_frames):
    fa,fb,fc = example_frames
    p = Point([1,1,3])
    pt = p.express_in_frame(fc)
    assert pt==Point([0,0,1])

def test_vector_express_in_frame(example_frames):
    fa,fb,fc = example_frames
    v = Vector([1,1,3])
    vt = v.express_in_frame(fc)
    assert vt==Vector([math.sqrt(2),0,-3])

def test_vector_express_in_frame_with_original_frame(example_frames):
    fa,fb,fc = example_frames
    v = Vector([1,1,3])
    vt = v.express_in_frame(fc, original_frame=fa)
    assert vt==Vector([1,1,3])

def test_point_transform(example_frames):
    fa,fb,fc = example_frames
    p = Point([1,1,3])
    pt = p.transform(fc)
    assert pt==Point([1+math.sqrt(2),1,1])

def test_vector_transform(example_frames):
    fa,fb,fc = example_frames
    v = Vector([1,1,3])
    vt = v.transform(fc)
    assert vt==Vector([math.sqrt(2),0,-3])
